using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;

namespace lytest
{
    public class HandParserBuiltins
    {
        private static int builtin_strlen(string arg)
        {
            return arg.Length;
        }

        private static string builtin_strrepeat(string arg, int times)
        {
            return new StringBuilder().Insert(0, arg, times).ToString();
        }
        
        //////////////////////////////// End of builtin functions ////////////////////////////////
        public static HandParser.EvaluationResultType evaluate(string funcname, List<HandParser.EvaluationResultType> args)
        {
            var methodInfo = typeof(HandParserBuiltins).GetMethod("builtin_" + funcname, BindingFlags.NonPublic | BindingFlags.Static);
            object[] args_to_forward = new object[methodInfo.GetParameters().Length];
            for (var i = 0; i < args_to_forward.Length; ++i)
            {
                var debug = methodInfo.GetParameters()[i].ParameterType;
                var requiredType = methodInfo.GetParameters()[i].ParameterType.FullName;
                switch (requiredType)
                {
                    case "System.String":
                        if(args[i].type != HandParser.RuntimeType.STRING) 
                            throw new ArgumentException($"TypeChecking Error: expecting `string` at function argument, got {args[i].type}. at `{args[i].originalText}`");
                        args_to_forward[i] = args[i].valueString;
                        break;
                    case "System.Int32":
                        if(args[i].type != HandParser.RuntimeType.INT) 
                            throw new ArgumentException($"TypeChecking Error: expecting `int` at function argument, got {args[i].type}. at `{args[i].originalText}`");
                        args_to_forward[i] = args[i].valueInt;
                        break;
                    default:
                        throw new InvalidProgramException("builtin function can only accept supported type: int or string.");
                }
            }
            var call_result = methodInfo.Invoke(null, args_to_forward);
            var evaluation_result = new HandParser.EvaluationResultType();
            switch (methodInfo.ReturnType.FullName)
            {
                case "System.Int32":
                    evaluation_result.type = HandParser.RuntimeType.INT;
                    evaluation_result.valueInt = (int)call_result;
                    break;
                case "System.String":
                    evaluation_result.type = HandParser.RuntimeType.STRING;
                    evaluation_result.valueString = (string)call_result;
                    break;      
                default:
                    throw new InvalidProgramException("builtin function can only return supported type: int or string.");
            }
            return evaluation_result;
        }
    }
}